import 'package:analyzer/dart/element/element.dart';
import 'package:build/build.dart';
import 'package:source_gen/source_gen.dart';

import '../annotations.dart';

class ReduxActionHandlerGenerator2 extends Generator {
  @override
  String generate(LibraryReader library, BuildStep buildStep) {
    return '''
// hello, world
    ''';
  }
}

class ReduxActionHandlerGenerator extends GeneratorForAnnotation<ReduxActionHandler> {
  @override
  String generateForAnnotatedElement(
    Element element,
    ConstantReader annotation,
    BuildStep buildStep,
  ) {
    // 生成的 ReduxBloc 类名
    var className = annotation.read('type').typeValue.getDisplayString(withNullability: false);
    var newClassName = className.substring(0, className.length - 1);
    print('====  className $className');

    // Action 处理器类名
    var handlerClassName = element.name;
    print('====  handlerClassName: $handlerClassName');

    // 对应 ReduxBloc 的 PublicState 和 PrivateState 类名
    var publicState = '###PublicState###';
    var privateState = '###PrivateState###';

    var reduxBlocClassElement = annotation.read('type').typeValue.element;
    if (reduxBlocClassElement is ClassElement) {
      var typeArguments = reduxBlocClassElement.allSupertypes.first.typeArguments;
      publicState = typeArguments[0].getDisplayString(withNullability: false);
      privateState = typeArguments[1].getDisplayString(withNullability: false);
    }
    print('====  publicState: $publicState, privateState: $privateState');

    // 方法对应的 if else 语句
    var reducerSource = StringBuffer();
    var privateReducerSource = StringBuffer();
    var middlewareSource = StringBuffer();
    var afterwareSource = StringBuffer();

    if (element is ClassElement) {
      bool isReducerFirst = true;
      bool isPrivateReducerFirst = true;
      bool isMiddlewareFirst = true;
      bool isAfterwareFirst = true;

      for (final methodElement in element.methods) {
        var metadataList = methodElement.metadata;

        if (metadataList.isNotEmpty) {
          var methodAnnotation = metadataList.first;
          var methodAnnotationReader = ConstantReader(methodAnnotation.computeConstantValue());

          var methodName = methodElement.name;
          var annotationName = methodAnnotation.element?.displayName;
          var actionName = methodAnnotationReader.read('type').typeValue.getDisplayString(withNullability: false);
          var isCheck = methodAnnotationReader.peek('check')?.boolValue ?? false;
          var isAsync = methodAnnotationReader.peek('async')?.boolValue ?? true;
          print('====  methodName: $methodName, annotationName: $annotationName, actionName: $actionName, check: $isCheck. async: $isAsync');

          // Reducer
          if (annotationName == 'Reducer') {
            if (!isReducerFirst) {
              reducerSource.write('else ');
            }
            isReducerFirst = false;

            reducerSource.write('''
if (action is $actionName) {
  newState = handler.$methodName(action, newState, privateState);
}
''');
          }

          // PrivateReducer
          else if (annotationName == 'PrivateReducer') {
            if (!isPrivateReducerFirst) {
              privateReducerSource.write('else ');
            }
            isPrivateReducerFirst = false;

            privateReducerSource.write('''
if (action is $actionName) {
  newState = handler.$methodName(action, state, newState);
}
''');
          }

          // Middleware
          else if (annotationName == 'Middleware') {
            if (!isMiddlewareFirst) {
              middlewareSource.write('else ');
            }
            isMiddlewareFirst = false;

            middlewareSource.write('''
if (action is $actionName) {
''');
            if (isAsync) {
              middlewareSource.write('''
  () async {
''');
            }

            if (isCheck) {
              middlewareSource.write('''
    if (handler.check(state)) {
''');
            }

            middlewareSource.write('''
      handler.$methodName(dispatcher, state, action);
''');

            if (isCheck) {
              middlewareSource.write('''
    }
''');
            }

            if (isAsync) {
              middlewareSource.write('''
  }();
''');
            }

            middlewareSource.write('''
}
''');
          }

          // Afterware
          else if (annotationName == 'Afterware') {
            if (!isAfterwareFirst) {
              afterwareSource.write('else ');
            }
            isAfterwareFirst = false;

            afterwareSource.write('''
if (action is $actionName) {
''');
            if (isAsync) {
              afterwareSource.write('''
  () async {
''');
            }

            if (isCheck) {
              afterwareSource.write('''
    if (handler.check(state)) {
''');
            }

            afterwareSource.write('''
      handler.$methodName(dispatcher, state, action);
''');

            if (isCheck) {
              afterwareSource.write('''
    }
''');
            }

            if (isAsync) {
              afterwareSource.write('''
  }(); 
''');
            }

            afterwareSource.write('''
}
''');
          }
        }
      }
    }

    // print('==== reducer: \n${reducerSource.toString()}');
    // print('==== privateReducer: \n${privateReducerSource.toString()}');
    // print('==== middleware: \n${middlewareSource.toString()}');
    // print('==== afterware: \n${afterwareSource.toString()}');

    return '''
class $newClassName extends $className {
  $newClassName() {
    handler = $handlerClassName(this);
  }

  late $handlerClassName handler;

  @override
  $publicState reducer(ReduxAction action, $publicState state, $privateState privateState) {
    $publicState newState = state;

${reducerSource.toString()}

    return newState;
  }

  @override
  $privateState privateReducer(ReduxAction action, $publicState state, $privateState privateState) {
    $privateState newState = privateState;

${privateReducerSource.toString()}

    return newState;
  }

  @override
  FutureOr<ReduxAction> middleware(DispatchFunction dispatcher, ReduxState state, ReduxAction action) {

${middlewareSource.toString()}
      
    return action;
  }
}
    ''';
  }
}
